package com.chinaitop.depot.intelligent.basicData.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.basicData.model.TFumigationRecord;
import com.chinaitop.depot.intelligent.basicData.param.TFumigationRecordPageParam;
import com.chinaitop.depot.intelligent.basicData.param.TFumigationRecordParam;
import com.chinaitop.depot.intelligent.basicData.service.TFumigationRecordService;
import com.chinaitop.depot.intelligent.basicData.vo.TFumigationRecordAndAuditTimeVO;
import com.chinaitop.depot.intelligent.basicData.vo.TFumigationRecordVO;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 熏蒸备案 前端控制器
 * </p>
 *
 * @author root
 * @since 2024-05-10
 */
@RestController
@RequestMapping("/t-fumigationRecord")
@Api(value = "TFumigationRecordController", description = "熏蒸备案设备控制类")
public class TFumigationRecordController {


    @Autowired
    private TFumigationRecordService tFumigationRecordService;

    @GetMapping("/getList")
    @ApiOperation(value = "查询 熏蒸备案 信息列表", notes = "查询 通风设备 信息列表")
    public ResponseEntity<List<TFumigationRecordAndAuditTimeVO>> getList(TFumigationRecordPageParam pageParam) {

        return ResponseEntity.ok(tFumigationRecordService.getList(pageParam));
    }
    @PostMapping("/save")
    @ApiOperation(value = "熏蒸备案新增", notes = "新增")
    public ResponseEntity save( @RequestBody TFumigationRecordParam recordParam) {
        return ResponseEntity.ok(tFumigationRecordService.saveData(recordParam));
    }

    @PutMapping("/update")
    @ApiOperation(value = "熏蒸备案修改", notes = "修改")
    public ResponseEntity update( @RequestBody TFumigationRecordParam recordParam) {
        return ResponseEntity.ok(tFumigationRecordService.updateData(recordParam));
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "熏蒸备案删除", notes = "id删除")
    public ResponseEntity delete(@PathVariable("id") String id) {
        return ResponseEntity.ok(tFumigationRecordService.deleteData(id));
    }

    @GetMapping("/getById/{id}")
    @ApiOperation(value = "熏蒸备案详情", notes = "id获取详情")
    public ResponseEntity<TFumigationRecord> getById(@PathVariable("id") String id) {
        return ResponseEntity.ok(tFumigationRecordService.getById(id));
    }

    @GetMapping("/getInfo")
    @ApiOperation(value = "带出熏蒸作业库区数据详情", notes = "获取带出数据")
    public ResponseEntity<TFumigationRecordVO> getInfo(String orgId) {
        return ResponseEntity.ok(tFumigationRecordService.getInfo(orgId));
    }


    @GetMapping("/getStorehouseAndLqInfo")
    @ApiOperation(value = "带出仓房和粮情数据", notes = "带出仓房和粮情数据")
    public ResponseEntity<JSONObject> getLqInfo(String orgId, String storehouseCode, Integer warehouseId) {
        return ResponseEntity.ok(tFumigationRecordService.getStorehouseAndLqInfo(orgId,storehouseCode, warehouseId));
    }
}
