package com.chinaitop.depot.intelligent.basicData.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.basicData.mapper.OrgInfoMapper;
import com.chinaitop.depot.intelligent.basicData.mapper.TJctableMapper;
import com.chinaitop.depot.intelligent.basicData.mapper.UserInfoMapper;
import com.chinaitop.depot.intelligent.basicData.model.OrgInfo;
import com.chinaitop.depot.intelligent.basicData.model.TJctable;
import com.chinaitop.depot.intelligent.basicData.model.UserInfo;
import com.chinaitop.depot.intelligent.basicData.param.TJctablePageParam;
import com.chinaitop.depot.intelligent.basicData.param.TJctableParam;
import com.chinaitop.depot.intelligent.basicData.service.TJctableService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 数量监测设备配置 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-10
 */
@Service
public class TJctableServiceImpl extends ServiceImpl<TJctableMapper, TJctable> implements TJctableService {


    @Autowired
    private TJctableMapper tJctableMapper;

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Override
    public Page<TJctable> getList(TJctablePageParam pageParam) {

        Page<TJctable> page = new Page<>(pageParam.getPageNum(), pageParam.getPageSize());
        LambdaQueryWrapper<TJctable> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(TJctable::getOrgId, pageParam.getOrgId());
        if (StringUtils.isNotEmpty(pageParam.getVCfName())) {
            wrapper.like(TJctable::getVCfName, pageParam.getVCfName());
        }

        Page<TJctable> tJctablePage = tJctableMapper.selectPage(page, wrapper);
        List<TJctable> tJctableList = tJctablePage.getRecords();
        if (CollectionUtils.isNotEmpty(tJctableList)) {
            for (TJctable tJctable : tJctableList) {
                UserInfo userInfo = userInfoMapper.selectById(tJctable.getUpdatePerson());
                if (ObjectUtil.isNotEmpty(userInfo)) {
                    tJctable.setUpdatePerson(userInfo.getRealName());
                }
            }
        }
        return tJctablePage;
    }

    @Override
    public int saveData(TJctableParam param) {
        TJctable tJctable = new TJctable();
        BeanUtils.copyProperties(param, tJctable);
        tJctable.setId(UuidUtils.getCode());
        tJctable.setUpdatetime(new Date());
        return tJctableMapper.insert(tJctable);
    }

    @Override
    public int updateData(TJctableParam param) {
        TJctable tJctable = new TJctable();
        BeanUtils.copyProperties(param, tJctable);

        tJctable.setUpdatetime(new Date());
        return tJctableMapper.updateById(tJctable);
    }

    @Override
    public int deleteData(String id) {
        return tJctableMapper.deleteById(id);
    }


}
