package com.chinaitop.depot.intelligent.basicData.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.basicData.mapper.TLqinfoMapper;
import com.chinaitop.depot.intelligent.basicData.model.TFumigationCirculation;
import com.chinaitop.depot.intelligent.basicData.model.TLqinfo;
import com.chinaitop.depot.intelligent.basicData.param.*;
import com.chinaitop.depot.intelligent.basicData.service.TLqinfoService;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.UUID;


/**
 * <p>
 * 粮情设备配置 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-10
 */
@Service
public class TLqinfoServiceImpl extends ServiceImpl<TLqinfoMapper, TLqinfo> implements TLqinfoService {

    @Autowired
    private TLqinfoMapper tLqinfoMapper;

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 查询 粮情设备配置 信息列表
     */
    @Override
    public Page<TLqinfo> getList(TLqinfoToPageParam pageParam) {

        Page<TLqinfo> page = new Page<>(pageParam.getPageNum(), pageParam.getPageSize());

        LambdaQueryWrapper<TLqinfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(TLqinfo::getOrgId,pageParam.getOrgId());

        if (StringUtils.isNotEmpty(pageParam.getVcfcode())) {
            wrapper.eq(TLqinfo::getVcfcode, pageParam.getVcfcode());
        }
        return tLqinfoMapper.selectPage(page, wrapper);
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 粮情设备配置新增
     */
    @Override
    public int saveData(TLqinfoParam param) {

        TLqinfo tLqinfo = new TLqinfo();
        BeanUtils.copyProperties(param,tLqinfo);

        tLqinfo.setId(UuidUtils.getCode());

        return tLqinfoMapper.insert(tLqinfo);

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 粮情设备配置修改
     */
    @Override
    public int updateData(TLqinfoParam param) {

        TLqinfo tLqinfo = new TLqinfo();
        BeanUtils.copyProperties(param,tLqinfo);

        return tLqinfoMapper.updateById(tLqinfo);
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: |粮情设备配置删除
     */
    @Override
    public int deleteData(String id) {
        return tLqinfoMapper.deleteById(id);

    }
}
