package com.chinaitop.depot.intelligent.basicData.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.basicData.mapper.TVentilatioInfoMapper;
import com.chinaitop.depot.intelligent.basicData.model.TVentilatioInfo;
import com.chinaitop.depot.intelligent.basicData.param.TVentilatioInfoPageParam;
import com.chinaitop.depot.intelligent.basicData.param.TVentilatioInfoParam;
import com.chinaitop.depot.intelligent.basicData.service.TVentilatioInfoService;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicEnumMapper;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 通风设备配置 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-09
 */
@Service
public class TVentilatioInfoServiceImpl extends ServiceImpl<TVentilatioInfoMapper, TVentilatioInfo> implements TVentilatioInfoService {


    @Autowired
    private TVentilatioInfoMapper tVentilatioInfoMapper;

    @Autowired
    private BasicEnumMapper basicEnumMapper;

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 查询 通风设备 信息列表
     */
    @Override
    public Page<TVentilatioInfo> getList(TVentilatioInfoPageParam pageParam) {


        LambdaQueryWrapper<TVentilatioInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(TVentilatioInfo::getOrgId,pageParam.getOrgId());
        if (StringUtils.isNotEmpty(pageParam.getVdevname())) {
            wrapper.like(TVentilatioInfo::getVdevname, pageParam.getVdevname());
        }
        if (StringUtils.isNotEmpty(pageParam.getOrgName())) {
            wrapper.like(TVentilatioInfo::getOrgName, pageParam.getOrgName());
        }

        return tVentilatioInfoMapper.selectPage(new Page<>(pageParam.getPageNum(), pageParam.getPageSize()), wrapper);

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 通风设备新增
     */
    @Override
    public int saveData(TVentilatioInfoParam infoParam) {

        TVentilatioInfo tVentilatioInfo = new TVentilatioInfo();
        BeanUtils.copyProperties(infoParam, tVentilatioInfo);
        tVentilatioInfo.setId(UuidUtils.getCode());
        return tVentilatioInfoMapper.insert(tVentilatioInfo);

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 通风设备修改
     */
    @Override
    public int updateData(TVentilatioInfoParam infoParam) {
        TVentilatioInfo tVentilatioInfo = new TVentilatioInfo();
        BeanUtils.copyProperties(infoParam, tVentilatioInfo);

        return tVentilatioInfoMapper.updateById(tVentilatioInfo);

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 通风设备删除
     */
    @Override
    public int deleteData(String id) {

        return tVentilatioInfoMapper.deleteById(id);
    }
}
