package com.chinaitop.depot.intelligent.devData.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.basicData.mapper.TStationMapper;
import com.chinaitop.depot.intelligent.basicData.model.TStation;
import com.chinaitop.depot.intelligent.basicData.model.TStationSelect;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import com.chinaitop.depot.intelligent.devData.model.TLqData;
import com.chinaitop.depot.intelligent.devData.mapper.TLqDataMapper;
import com.chinaitop.depot.intelligent.devData.model.TLqDataPageParam;
import com.chinaitop.depot.intelligent.devData.service.TLqDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 多功能粮情-主表 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-10
 */
@Slf4j
@Service
public class TLqDataServiceImpl extends ServiceImpl<TLqDataMapper, TLqData> implements TLqDataService {
    @Resource
    private TLqDataMapper lqDataMapper;
    @Resource
    private TStationMapper stationMapper;
    @Resource
    private RestTemplate restTemplate;

    @Override
    public Page<TLqData> selectPageList(TLqDataPageParam param) {
        QueryWrapper<TLqData> wrapper = new QueryWrapper<>();
        wrapper.eq("iLq_yq", param.getIlqYq());
        wrapper.eq("org_id", param.getOrgId());
        if (StringUtils.isNotEmpty(param.getStorehouse()))
            wrapper.eq("storehouse", param.getStorehouse());
        Date gatherTime = param.getGatherTime();
        if (ObjectUtils.isNotEmpty(gatherTime))
            wrapper.between("gather_time", DateUtils.formatDate(gatherTime) + DateUtils.startTime, DateUtils.formatDate(gatherTime) + DateUtils.endTime);
        wrapper.orderByDesc("gather_time");
        return lqDataMapper.selectPage(new Page<>(param.getCurrent(), param.getSize()), wrapper);
    }

    @Override
    public ResponseEntity sendStoreDetection(TStationSelect param) {
        // 查询发送指令地址
        try {
            QueryWrapper<TStation> wrapper = new QueryWrapper<>();
            wrapper.eq("islocal", 1);
            String orgId = param.getOrgId();
            if (StringUtils.isEmpty(orgId)) { // 静态定时任务所有库检测
                List<TStation> tStations = stationMapper.selectList(wrapper);
                if (CollectionUtils.isNotEmpty(tStations)) {
                    tStations.forEach(station -> this.restHttp(param, station.getServiceip()));
                }
            } else { // 单库/选仓检测
                wrapper.eq("org_id", param.getOrgId());
                TStation tStation = stationMapper.selectOne(wrapper);
                if (ObjectUtils.isNotEmpty(tStation)) {
                    String serviceIp = tStation.getServiceip();
                    this.restHttp(param, serviceIp);
                }
            }
        } catch (RestClientException e) {
            e.printStackTrace();
            return ResponseEntity.failed("指令发送失败!");
        }
        return ResponseEntity.ok();
    }

    // 发送检测指令
    private void restHttp(TStationSelect param, String serviceIp) {
        if (param.getIlqYq().equals("0")) {
            JSONObject object = new JSONObject();
            object.put("cmd", "fetch_data");
            if (StringUtils.isEmpty(param.getStorehouseCode()))
                object.put("data", ""); // 全库检测发送空字符串
            else
                object.put("data", param.getStorehouseCode());
            JSONObject result = restTemplate.postForObject(serviceIp, object, JSONObject.class);
            log.info("sendStoreDetection: {}", result);
        }
    }
}
