package com.chinaitop.depot.intelligent.financialSupervision.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessFsAccount;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessFsStatisticalAccount;
import com.chinaitop.depot.intelligent.financialSupervision.param.*;
import com.chinaitop.depot.intelligent.financialSupervision.service.BusinessFsAccountService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 财务监管-会计账 前端控制器
 * </p>
 *
 * @author root
 * @since 2024-07-02
 */
@RestController
@RequestMapping("/business-fs-account")
@Api(value = "BusinessFsAccount", description = "会计账控制类")
public class BusinessFsAccountController {

    @Autowired
    private BusinessFsAccountService businessFsAccountService;

    @GetMapping("/getList")
    @ApiOperation(value = "查询 会计账 信息列表", notes = "查询 会计账 信息列表")
    public ResponseEntity<Page<BusinessFsAccount>> getList(@Validated BusinessFsAccountPageParam pageParam) {

        return ResponseEntity.ok(businessFsAccountService.getList(pageParam));
    }

    @PostMapping("/save")
    @ApiOperation(value = "会计账新增", notes = "新增")
    public ResponseEntity save(@Validated @RequestBody BusinessFsAccountAddParam param) {
        return ResponseEntity.ok(businessFsAccountService.saveData(param));
    }

    @PutMapping("/update")
    @ApiOperation(value = "会计账修改", notes = "修改")
    public ResponseEntity update(@Validated @RequestBody BusinessFsAccountUpdateParam param) {
        return ResponseEntity.ok(businessFsAccountService.updateData(param));
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "会计账删除", notes = "id删除")
    public ResponseEntity delete(@PathVariable("id") Integer id) {
        return ResponseEntity.ok(businessFsAccountService.deleteData(id));
    }

    @GetMapping("/getById/{id}")
    @ApiOperation(value = "会计账详情", notes = "id获取详情")
    public ResponseEntity<BusinessFsAccount> getById(@PathVariable("id") String id) {
        return ResponseEntity.ok(businessFsAccountService.getDataById(id));
    }
}
