package com.chinaitop.depot.intelligent.financialSupervision.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessContract;
import com.chinaitop.depot.intelligent.financialSupervision.param.*;
import com.chinaitop.depot.intelligent.financialSupervision.service.BusinessFsInConfirmService;
import com.chinaitop.depot.intelligent.financialSupervision.vo.BusinessFsInConfirmDetailVO;
import com.chinaitop.depot.intelligent.financialSupervision.vo.BusinessFsInConfirmPageVO;
import com.chinaitop.depot.intelligent.financialSupervision.vo.GetFsConfirmByContractIdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 财务监管-入库确认单 前端控制器
 * </p>
 *
 * @author root
 * @since 2024-07-04
 */
@RestController
@RequestMapping("/business-fs-in-confirm")
@Api(value = "BusinessFsInConfirm", description = "入库确认单控制类")
public class BusinessFsInConfirmController {

    @Autowired
    private BusinessFsInConfirmService businessFsInConfirmService;


    @GetMapping("/getList")
    @ApiOperation(value = "查询 入库确认单 信息列表", notes = "查询 入库确认单 信息列表")
    public ResponseEntity<Page<BusinessFsInConfirmPageVO>> getList(@Validated BusinessFsInConfirmPageParam pageParam) {

        return ResponseEntity.ok(businessFsInConfirmService.getList(pageParam));
    }

    @PostMapping("/save")
    @ApiOperation(value = "入库确认单新增", notes = "新增")
    public ResponseEntity save(@Validated @RequestBody BusinessFsInConfirmAddParam param) {
        return ResponseEntity.ok(businessFsInConfirmService.saveData(param));
    }
    @PutMapping("/update")
    @ApiOperation(value = "入库确认单修改", notes = "修改")
    public ResponseEntity update(@Validated @RequestBody BusinessFsInConfirmUpdateParam param) {
        return ResponseEntity.ok(businessFsInConfirmService.updateData(param));
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "入库确认单删除", notes = "id删除")
    public ResponseEntity delete(@PathVariable("id") Integer id) {
        return ResponseEntity.ok(businessFsInConfirmService.deleteData(id));
    }

    @GetMapping("/getById/{id}")
    @ApiOperation(value = "入库确认单详情", notes = "id获取详情")
    public ResponseEntity<BusinessFsInConfirmDetailVO> getById(@PathVariable("id") String id) {
        return ResponseEntity.ok(businessFsInConfirmService.getDataById(id));
    }

    @GetMapping("/getContractData")
    @ApiOperation(value = "获取采购合同编号", notes = "获取采购合同编号")
    public ResponseEntity<List<BusinessContract>> getContractData(Integer orgId) {

        return ResponseEntity.ok(businessFsInConfirmService.getContractData(orgId));
    }
    @GetMapping("/getByContractId/{id}")
    @ApiOperation(value = "通过合同id获取带出数据", notes = "通过合同id获取带出数据")
    public ResponseEntity<GetFsConfirmByContractIdVO> getByContractId(@PathVariable("id") String id) {
        return ResponseEntity.ok(businessFsInConfirmService.getByContractId(id));
    }



}
