package com.chinaitop.depot.intelligent.financialSupervision.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.basicData.model.TCcinfo;
import com.chinaitop.depot.intelligent.basicData.param.TLqinfoPageParam;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessFsCostStandard;
import com.chinaitop.depot.intelligent.financialSupervision.mapper.BusinessFsCostStandardMapper;
import com.chinaitop.depot.intelligent.financialSupervision.param.BusinessFsCostStandardAddParam;
import com.chinaitop.depot.intelligent.financialSupervision.param.BusinessFsCostStandardUpdateParam;
import com.chinaitop.depot.intelligent.financialSupervision.param.FsCostStandardPageParam;
import com.chinaitop.depot.intelligent.financialSupervision.service.BusinessFsCostStandardService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.financialSupervision.vo.BusinessFsCostStandardVO;
import com.chinaitop.depot.intelligent.financialSupervision.vo.GetGrainVarietyVO;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicEnumMapper;
import com.chinaitop.depot.intelligent.grainReserves.model.BasicEnum;
import com.thoughtworks.xstream.mapper.EnumMapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 财务监管-费用标准维护 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-06-27
 */
@Service
public class BusinessFsCostStandardServiceImpl extends ServiceImpl<BusinessFsCostStandardMapper, BusinessFsCostStandard> implements BusinessFsCostStandardService {

    @Autowired
    private BusinessFsCostStandardMapper businessFsCostStandardMapper;

    @Autowired
    private BasicEnumMapper basicEnumMapper;

    /**
     * 查询 费用标准维护 信息列表
     *
     * @author hjc
     * @date 2024/6/27 10:09
     */
    @Override
    public Page<BusinessFsCostStandard> getList(FsCostStandardPageParam pageParam) {
        Page<BusinessFsCostStandard> page = new Page<>(pageParam.getPageNum(), pageParam.getPageSize());
        LambdaQueryWrapper<BusinessFsCostStandard> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(BusinessFsCostStandard::getOrgId, pageParam.getOrgId());
        if (ObjectUtil.isNotEmpty(pageParam.getStandardType())) {
            wrapper.eq(BusinessFsCostStandard::getStandardType, pageParam.getStandardType());
        }
        return businessFsCostStandardMapper.selectPage(page, wrapper);
    }


    /**
     * 费用标准维护新增
     *
     * @author hjc
     * @date 2024/6/27 10:22
     */
    @Override
    public int saveData(BusinessFsCostStandardAddParam param) {

        BusinessFsCostStandard businessFsCostStandard = new BusinessFsCostStandard();
        BeanUtils.copyProperties(param, businessFsCostStandard);

        businessFsCostStandard.setCreatedate(new Date());
        businessFsCostStandard.setUpdatedate(new Date());

        return businessFsCostStandardMapper.insert(businessFsCostStandard);

    }


    /**
     * 费用标准维护修改
     *
     * @author hjc
     * @date 2024/6/27 10:28
     */
    @Override
    public int updateData(BusinessFsCostStandardUpdateParam param) {
        BusinessFsCostStandard businessFsCostStandard = new BusinessFsCostStandard();
        BeanUtils.copyProperties(param, businessFsCostStandard);

        businessFsCostStandard.setUpdatedate(new Date());

        return businessFsCostStandardMapper.updateById(businessFsCostStandard);
    }


    /**
     * 费用标准维护删除
     *
     * @author hjc
     * @date 2024/6/27 10:31
     */
    @Override
    public int deleteData(Integer id) {
        return businessFsCostStandardMapper.deleteById(id);
    }

    /**
     * 费用标准维护详情
     *
     * @author hjc
     * @date 2024/6/27 10:56
     */
    @Override
    public BusinessFsCostStandardVO getDataById(String id) {
        BusinessFsCostStandardVO vo = new BusinessFsCostStandardVO();
        BusinessFsCostStandard businessFsCostStandard = businessFsCostStandardMapper.selectById(id);
        BeanUtils.copyProperties(businessFsCostStandard,vo);
        //获取种类
        BasicEnum grainType = basicEnumMapper.selectById(businessFsCostStandard.getGrainTypeId());
        vo.setGrainTypeName(grainType.getEnumname());

        //获取品种
        BasicEnum grainVariety = basicEnumMapper.selectById(businessFsCostStandard.getGrainVarietyId());
        vo.setGrainVarietyName(grainVariety.getEnumname());

        return vo;
    }

    /**
     * 获取粮食品种
     *
     * @author hjc
     * @date 2024/6/27 10:39
     */
    @Override
    public List<GetGrainVarietyVO> getGrainVariety() {
        List<BasicEnum> basicEnums = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                .eq(BasicEnum::getParentid, 1061));

        List<GetGrainVarietyVO> list = new ArrayList<>();

        for (BasicEnum basicEnum : basicEnums) {
            GetGrainVarietyVO vo = new GetGrainVarietyVO();
            vo.setGrainVarietyId(basicEnum.getEnumid());
            vo.setGrainVarietyName(basicEnum.getEnumname());
            list.add(vo);
        }
        return list;
    }
}
