package com.chinaitop.depot.intelligent.financialSupervision.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.basicData.mapper.BasicStorehouseMapper;
import com.chinaitop.depot.intelligent.basicData.model.BasicStorehouse;
import com.chinaitop.depot.intelligent.basicData.param.StorehouseAndLqVO;
import com.chinaitop.depot.intelligent.financialSupervision.mapper.*;
import com.chinaitop.depot.intelligent.financialSupervision.model.*;
import com.chinaitop.depot.intelligent.financialSupervision.param.BusinessFsOutConfirmAddParam;
import com.chinaitop.depot.intelligent.financialSupervision.param.BusinessFsOutConfirmPageParam;
import com.chinaitop.depot.intelligent.financialSupervision.param.BusinessFsOutConfirmUpdateParam;
import com.chinaitop.depot.intelligent.financialSupervision.service.BusinessFsOutConfirmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.financialSupervision.vo.*;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicEnumMapper;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.intelligent.grainReserves.model.BasicEnum;
import com.chinaitop.depot.intelligent.grainReserves.model.BasicWarehouse;
import com.chinaitop.depot.intelligent.utils.OrderNumberUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 财务监管-销售收入上缴-出库验收确认 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-07-23
 */
@Service
public class BusinessFsOutConfirmServiceImpl extends ServiceImpl<BusinessFsOutConfirmMapper, BusinessFsOutConfirm> implements BusinessFsOutConfirmService {

    @Autowired
    private BusinessFsOutConfirmMapper businessFsOutConfirmMapper;

    @Autowired
    private BusinessContractMapper businessContractMapper;

    @Autowired
    private BasicEnumMapper basicEnumMapper;

    @Autowired
    private BusinessStoreWareDetailMapper businessStoreWareDetailMapper;

    @Autowired
    private BusinessFsOutConfirmContractItemMapper businessFsOutConfirmContractItemMapper;

    @Autowired
    private BusinessFsOutConfirmDiffPriceItemMapper businessFsOutConfirmDiffPriceItemMapper;

    @Autowired
    private BasicStorehouseMapper basicStorehouseMapper;

    @Autowired
    private BasicWarehouseMapper basicWarehouseMapper;

    /**
     * 获取销售合同编号
     *
     * @author hjc
     * @date 2024/7/23 16:56
     */
    @Override
    public List<BusinessContract> getContractData(Integer orgId) {
        return businessContractMapper.selectList(new LambdaQueryWrapper<BusinessContract>()
                .eq(BusinessContract::getAuditState, 2)
                .eq(BusinessContract::getContractType, "3148")
                .eq(BusinessContract::getOrgId, orgId));
    }

    /**
     * 通过合同id获取带出数据
     *
     * @author hjc
     * @date 2024/7/23 16:37
     */
    @Override
    public GetFsOutConfirmByContractVO getByContractId(String id) {

        GetFsOutConfirmByContractVO vo = new GetFsOutConfirmByContractVO();
        //获取合同信息
        BusinessContract businessContract = businessContractMapper.selectById(id);
        if (ObjectUtils.isNotEmpty(businessContract)) {
            vo.setContractNumber(businessContract.getContractNumber());
            vo.setContractTitle(businessContract.getContractTitle());
            vo.setPurchaseUnit(businessContract.getPurchaseUnit());
            BasicEnum basicEnum = basicEnumMapper.selectById(businessContract.getContractType());
            if (ObjectUtils.isNotEmpty(basicEnum)) {
                vo.setContractType(basicEnum.getEnumname());
            }

        }
        //获取合同详情信息
        List<BusinessStoreWareDetail> storeWareDetailList = businessStoreWareDetailMapper.selectList(
                Wrappers.lambdaQuery(BusinessStoreWareDetail.class)
                        .eq(BusinessStoreWareDetail::getZid, id)
                        .eq(BusinessStoreWareDetail::getType, "contract"));

        List<BusinessStoreWareDetailDataVO> storeWareDetailVOList = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(storeWareDetailList)) {
            for (BusinessStoreWareDetail businessStoreWareDetail : storeWareDetailList) {
                BusinessStoreWareDetailDataVO storeWareDetailVO = new BusinessStoreWareDetailDataVO();

                storeWareDetailVO.setId(businessStoreWareDetail.getId());
                //获取仓房
                BasicEnum houseInfo = basicEnumMapper.selectById(businessStoreWareDetail.getHouseId());
                if (ObjectUtils.isNotEmpty(houseInfo)) {
                    storeWareDetailVO.setBasicStorehouseId(businessStoreWareDetail.getHouseId());
                    storeWareDetailVO.setBasicStorehouseName(houseInfo.getEnumname());
                }

                //获取货位
                BasicEnum warehouse = basicEnumMapper.selectById(businessStoreWareDetail.getWarehouseId());
                if (ObjectUtils.isNotEmpty(warehouse)) {
                    storeWareDetailVO.setWarehouseId(businessStoreWareDetail.getWarehouseId());
                    storeWareDetailVO.setWarehouseName(warehouse.getEnumname());
                }

                //获取品种
                BasicEnum grainKind = basicEnumMapper.selectById(businessStoreWareDetail.getGrainKind());
                if (ObjectUtils.isNotEmpty(grainKind)) {
                    storeWareDetailVO.setGrainVarietyId(businessStoreWareDetail.getGrainKind());
                    storeWareDetailVO.setGrainVarietyName(grainKind.getEnumname());
                }
                //获取性质
                BasicEnum grainAttribute = basicEnumMapper.selectById(businessStoreWareDetail.getGrainAttribute());
                if (ObjectUtils.isNotEmpty(grainAttribute)) {
                    storeWareDetailVO.setGrainNatureId(businessStoreWareDetail.getGrainAttribute());
                    storeWareDetailVO.setGrainNatureName(grainAttribute.getEnumname());
                }

                //销售合同使用轮出价格，数量和总价
                storeWareDetailVO.setSettlePrice(businessStoreWareDetail.getOutPrice() == null ? "0.0" : businessStoreWareDetail.getOutPrice());

                //获取保存的确认单数量
                Double sum = businessFsOutConfirmMapper.selectSum(id, businessStoreWareDetail.getGrainKind(), businessStoreWareDetail.getGrainAttribute());
                if (sum == null) {
                    sum = 0.0;
                }
                //用总的减去填写的数量
                if (businessStoreWareDetail.getOutCount() == null) {
                    businessStoreWareDetail.setOutCount("0.0");
                }
                double num = Double.parseDouble(businessStoreWareDetail.getOutCount()) - sum;

                if (num <= 0) {
                    storeWareDetailVO.setSettleQuantity("0.0");
                    storeWareDetailVOList.add(storeWareDetailVO);
                } else {
                    storeWareDetailVO.setSettleQuantity(Double.toString(num));
                    storeWareDetailVOList.add(storeWareDetailVO);
                }
            }
        }


        List<BusinessStoreWareDetailDataVO> list = new ArrayList<>();

        //通过仓房，货位，品种，性质进行汇总
        Map<String, List<BusinessStoreWareDetailDataVO>> collect =
                storeWareDetailVOList.stream().collect(Collectors.groupingBy(this::fetchGroupKey));


        Set<Map.Entry<String, List<BusinessStoreWareDetailDataVO>>> entries = collect.entrySet();

        for (Map.Entry<String, List<BusinessStoreWareDetailDataVO>> entry : entries) {
            int sum = 0;
            List<BusinessStoreWareDetailDataVO> value = entry.getValue();
            List<String> collect1 = value.stream().map(BusinessStoreWareDetailDataVO::getSettleQuantity).collect(Collectors.toList());
            for (String s : collect1) {
                sum += Double.parseDouble(s);
            }
            BusinessStoreWareDetailDataVO storeWareDetailVO = value.get(0);
            storeWareDetailVO.setSettleQuantity(String.valueOf(sum));
            list.add(storeWareDetailVO);
        }


        vo.setStoreWareDetailDataVOList(list);

        return vo;


    }


    private String fetchGroupKey(BusinessStoreWareDetailDataVO vo) {
        return vo.getBasicStorehouseId() + "_" + vo.getWarehouseId() + "_" + vo.getGrainVarietyId() + "_" + vo.getGrainNatureId();
    }

    /**
     * 出库验收确认详情
     *
     * @author hjc
     * @date 2024/7/24 9:46
     */
    @Override
    public BusinessFsOutConfirmDetailVO getDataById(String id) {

        BusinessFsOutConfirmDetailVO vo = new BusinessFsOutConfirmDetailVO();

        //通过id获取确认单
        BusinessFsOutConfirm businessFsOutConfirm = businessFsOutConfirmMapper.selectById(id);
        BeanUtils.copyProperties(businessFsOutConfirm, vo);

        //获取合同
        BusinessContract businessContract = businessContractMapper.selectById(businessFsOutConfirm.getContractId());
        BeanUtils.copyProperties(businessContract, vo);
        if (ObjectUtils.isNotEmpty(businessContract)){

            BasicEnum basicEnum = basicEnumMapper.selectById(businessContract.getContractType());
            vo.setContractType(basicEnum.getEnumname());
        }

        //通过确认单获取合同信息
        List<BusinessFsOutConfirmContractItem> businessFsOutConfirmContractItemList = businessFsOutConfirmContractItemMapper.selectList(new LambdaQueryWrapper<BusinessFsOutConfirmContractItem>()
                .eq(BusinessFsOutConfirmContractItem::getFsOutConfirmId, businessFsOutConfirm.getId()));
        List<BusinessStoreWareDetailDataVO> listVO = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(businessFsOutConfirmContractItemList)) {
            for (BusinessFsOutConfirmContractItem businessFsOutConfirmContractItem : businessFsOutConfirmContractItemList) {

                BusinessStoreWareDetailDataVO storeWareDetailVO = new BusinessStoreWareDetailDataVO();
                storeWareDetailVO.setId(businessFsOutConfirmContractItem.getId());
                storeWareDetailVO.setSettleQuantity(String.valueOf(businessFsOutConfirmContractItem.getSettleQuantity()));
                storeWareDetailVO.setSettlePrice(String.valueOf(businessFsOutConfirmContractItem.getSettlePrice()));
                storeWareDetailVO.setSettleAmount(String.valueOf(businessFsOutConfirmContractItem.getSettleAmount()));

                //设置仓房货位品种性质
                BasicStorehouse basicStorehouse = basicStorehouseMapper.selectById(businessFsOutConfirmContractItem.getBasicStorehouseId());
                if (ObjectUtils.isNotEmpty(basicStorehouse)) {
                    storeWareDetailVO.setBasicStorehouseId(businessFsOutConfirmContractItem.getBasicStorehouseId());
                    storeWareDetailVO.setBasicStorehouseName(basicStorehouse.getStorehouseName());
                }

                BasicWarehouse basicWarehouse = basicWarehouseMapper.selectById(businessFsOutConfirmContractItem.getWarehouseId());
                if (ObjectUtils.isNotEmpty(basicWarehouse)) {
                    storeWareDetailVO.setWarehouseId(businessFsOutConfirmContractItem.getWarehouseId());
                    storeWareDetailVO.setWarehouseName(basicWarehouse.getWarehouseName());
                }

                BasicEnum grainVariety = basicEnumMapper.selectById(businessFsOutConfirmContractItem.getGrainVarietyId());
                if (ObjectUtils.isNotEmpty(grainVariety)) {
                    storeWareDetailVO.setGrainVarietyName(grainVariety.getEnumname());
                    storeWareDetailVO.setGrainVarietyId(businessFsOutConfirmContractItem.getGrainVarietyId());
                }
                BasicEnum grainNature = basicEnumMapper.selectById(businessFsOutConfirmContractItem.getGrainNatureId());
                if (ObjectUtils.isNotEmpty(grainNature)) {
                    storeWareDetailVO.setGrainNatureId(businessFsOutConfirmContractItem.getGrainNatureId());
                    storeWareDetailVO.setGrainNatureName(grainNature.getEnumname());
                }
                listVO.add(storeWareDetailVO);

            }
        }
        vo.setStoreWareDetailDataVOList(listVO);

        List<BusinessFsOutConfirmDiffPriceItem> fsOutConfirmDiffPriceItemList = businessFsOutConfirmDiffPriceItemMapper.selectList(new LambdaQueryWrapper<BusinessFsOutConfirmDiffPriceItem>()
                .eq(BusinessFsOutConfirmDiffPriceItem::getFsOutConfirmId, businessFsOutConfirm.getId()));
        if (CollectionUtil.isNotEmpty(fsOutConfirmDiffPriceItemList)) {
            vo.setFsOutConfirmDiffPriceItemList(fsOutConfirmDiffPriceItemList);
        }

        return vo;

    }

    /**
     * 出库验收确认删除
     *
     * @author hjc
     * @date 2024/7/24 10:42
     */
    @Transactional
    @Override
    public int deleteData(Integer id) {
        //删除三张表数据
        businessFsOutConfirmMapper.deleteById(id);
        businessFsOutConfirmContractItemMapper.delete(new LambdaQueryWrapper<BusinessFsOutConfirmContractItem>()
                .eq(BusinessFsOutConfirmContractItem::getFsOutConfirmId, id));
        return businessFsOutConfirmDiffPriceItemMapper.delete(new LambdaQueryWrapper<BusinessFsOutConfirmDiffPriceItem>()
                .eq(BusinessFsOutConfirmDiffPriceItem::getFsOutConfirmId, id));
    }


    /**
     * 出库验收确认新增
     *
     * @author hjc
     * @date 2024/7/24 11:09
     */
    @Transactional
    @Override
    public int saveData(BusinessFsOutConfirmAddParam param) {
        //保存主表
        String outboundConfirmationNumber = businessFsOutConfirmMapper.selectOutboundConfirmationNumber();
        String s = OrderNumberUtil.initNum("CKQRD", 3, outboundConfirmationNumber);
        BusinessFsOutConfirm businessFsOutConfirm = new BusinessFsOutConfirm();
        BeanUtils.copyProperties(param, businessFsOutConfirm);
        businessFsOutConfirm.setCreatedate(new Date());
        businessFsOutConfirm.setUpdatedate(new Date());
        businessFsOutConfirm.setOutboundConfirmationNumber(s);
        businessFsOutConfirmMapper.insert(businessFsOutConfirm);

        //保存合同表
        List<BusinessFsOutConfirmContractItem> fsOutConfirmContractItemList = param.getFsOutConfirmContractItemList();
        if (CollectionUtil.isNotEmpty(fsOutConfirmContractItemList)) {
            for (BusinessFsOutConfirmContractItem businessFsOutConfirmContractItem : fsOutConfirmContractItemList) {
                businessFsOutConfirmContractItem.setCreatedate(new Date());
                businessFsOutConfirmContractItem.setUpdatedate(new Date());
                businessFsOutConfirmContractItem.setFsOutConfirmId(businessFsOutConfirm.getId());
                businessFsOutConfirmContractItemMapper.insert(businessFsOutConfirmContractItem);
            }

        }
        //保存差价上缴
        List<BusinessFsOutConfirmDiffPriceItem> fsOutConfirmDiffPriceItemList = param.getFsOutConfirmDiffPriceItemList();
        if (CollectionUtil.isNotEmpty(fsOutConfirmDiffPriceItemList)) {
            for (BusinessFsOutConfirmDiffPriceItem businessFsOutConfirmDiffPriceItem : fsOutConfirmDiffPriceItemList) {
                businessFsOutConfirmDiffPriceItem.setCreatedate(new Date());
                businessFsOutConfirmDiffPriceItem.setUpdatedate(new Date());
                businessFsOutConfirmDiffPriceItem.setFsOutConfirmId(businessFsOutConfirm.getId());
                businessFsOutConfirmDiffPriceItemMapper.insert(businessFsOutConfirmDiffPriceItem);
            }
        }

        return 1;
    }

    /**
     * 出库验收确认修改
     *
     * @author hjc
     * @date 2024/7/24 11:18
     */
    @Transactional
    @Override
    public int updateData(BusinessFsOutConfirmUpdateParam param) {

        //修改主表
        BusinessFsOutConfirm businessFsOutConfirm = new BusinessFsOutConfirm();
        BeanUtils.copyProperties(param, businessFsOutConfirm);
        businessFsOutConfirm.setUpdatedate(new Date());
        businessFsOutConfirmMapper.updateById(businessFsOutConfirm);

        //删除合同和差价上缴
        businessFsOutConfirmContractItemMapper.delete(new LambdaQueryWrapper<BusinessFsOutConfirmContractItem>()
                .eq(BusinessFsOutConfirmContractItem::getFsOutConfirmId, param.getId()));
        businessFsOutConfirmDiffPriceItemMapper.delete(new LambdaQueryWrapper<BusinessFsOutConfirmDiffPriceItem>()
                .eq(BusinessFsOutConfirmDiffPriceItem::getFsOutConfirmId, param.getId()));


        //保存合同表
        List<BusinessFsOutConfirmContractItem> fsOutConfirmContractItemList = param.getFsOutConfirmContractItemList();
        if (CollectionUtil.isNotEmpty(fsOutConfirmContractItemList)) {
            for (BusinessFsOutConfirmContractItem businessFsOutConfirmContractItem : fsOutConfirmContractItemList) {
                businessFsOutConfirmContractItem.setUpdatedate(new Date());
                businessFsOutConfirmContractItem.setFsOutConfirmId(param.getId());
                businessFsOutConfirmContractItemMapper.insert(businessFsOutConfirmContractItem);
            }

        }
        //保存差价上缴
        List<BusinessFsOutConfirmDiffPriceItem> fsOutConfirmDiffPriceItemList = param.getFsOutConfirmDiffPriceItemList();
        if (CollectionUtil.isNotEmpty(fsOutConfirmDiffPriceItemList)) {
            for (BusinessFsOutConfirmDiffPriceItem businessFsOutConfirmDiffPriceItem : fsOutConfirmDiffPriceItemList) {
                businessFsOutConfirmDiffPriceItem.setUpdatedate(new Date());
                businessFsOutConfirmDiffPriceItem.setFsOutConfirmId(param.getId());
                businessFsOutConfirmDiffPriceItemMapper.insert(businessFsOutConfirmDiffPriceItem);
            }
        }
        return 1;

    }

    /**
     * 查询 出库验收确认 信息列表
     *
     * @author hjc
     * @date 2024/7/24 11:25
     */
    @Override
    public Page<BusinessFsOutConfirmPageVO> getList(BusinessFsOutConfirmPageParam pageParam) {
        Page<BusinessFsOutConfirmPageVO> page = new Page<>(pageParam.getPageNum(), pageParam.getPageSize());
        return businessFsOutConfirmMapper.selectMyPage(page, pageParam.getContractNumber(),pageParam.getOrgId());
    }
}
