package com.chinaitop.depot.intelligent.grainReserves.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.chinaitop.depot.intelligent.basicData.mapper.BasicStorehouseMapper;
import com.chinaitop.depot.intelligent.basicData.mapper.OrgInfoMapper;
import com.chinaitop.depot.intelligent.basicData.model.OrgInfo;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicEnumMapper;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.intelligent.basicData.model.BasicStorehouse;
import com.chinaitop.depot.intelligent.grainReserves.mapper.GrainSituationCardMapper;
import com.chinaitop.depot.intelligent.grainReserves.model.BasicEnum;
import com.chinaitop.depot.intelligent.grainReserves.model.BasicWarehouse;
import com.chinaitop.depot.intelligent.grainReserves.mapper.BusinessGslrLossCollectMapper;
import com.chinaitop.depot.intelligent.grainReserves.model.BusinessGslrLossCollect;
import com.chinaitop.depot.intelligent.grainReserves.model.GrainSituationCard;
import com.chinaitop.depot.intelligent.grainReserves.param.BusinessGslrLossCollectPageParam;
import com.chinaitop.depot.intelligent.grainReserves.param.BusinessGslrLossCollectParam;
import com.chinaitop.depot.intelligent.grainReserves.service.BusinessGslrLossCollectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.grainReserves.vo.BusinessGslrLossCollectVO;
import com.chinaitop.depot.intelligent.grainReserves.vo.WarehouseAndInfoVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 粮食储备-节粮损益-损溢损失信息采集 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-15
 */
@Service
public class BusinessGslrLossCollectServiceImpl extends ServiceImpl<BusinessGslrLossCollectMapper, BusinessGslrLossCollect> implements BusinessGslrLossCollectService {
    private static Map<String, Integer> sequenceNumbers = new HashMap<>(); // 映射：仓房代码到顺序号
    private static final int MAX_SEQUENCE_NUMBER = 1000; // 顺序号的最大值
    private static final String DATE_FORMAT = "yyyyMMdd";
    private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern(DATE_FORMAT);

    @Autowired
    private BusinessGslrLossCollectMapper businessGslrLossCollectMapper;

    @Autowired
    private BasicWarehouseMapper basicWarehouseMapper;

    @Autowired
    private BasicStorehouseMapper basicStorehouseMapper;

    @Autowired
    private GrainSituationCardMapper grainSituationCardMapper;

    @Autowired
    private BasicEnumMapper basicEnumMapper;

    @Autowired
    private OrgInfoMapper orgInfoMapper;

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 查询 损溢损失信息采集 信息列表
     */
    @Override
    public List<BusinessGslrLossCollectVO> getList(BusinessGslrLossCollectPageParam pageParam) {
        //获取对应的数据
        List<BusinessGslrLossCollect> collectList = businessGslrLossCollectMapper.selectMyPage
                (pageParam.getCurrent(), pageParam.getSize(), pageParam.getGrainNatureName(),
                        pageParam.getGrainVarietyName(), pageParam.getOrgName(),pageParam.getOrgId());
        //封装返回对象
        List<BusinessGslrLossCollectVO> businessGslrLossCollectVOS = new ArrayList<>();
        for (BusinessGslrLossCollect businessGslrLossCollect : collectList) {
            BusinessGslrLossCollectVO businessGslrLossCollectVO = new BusinessGslrLossCollectVO();
            BeanUtils.copyProperties(businessGslrLossCollect, businessGslrLossCollectVO);
            //查询对应的库区名称
            OrgInfo orgInfo = orgInfoMapper.selectById(businessGslrLossCollect.getOrgId());
            businessGslrLossCollectVO.setOrgName(orgInfo.getOrgName());
            //查询对应的品种性质等级
            List<BasicEnum> basicEnums = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                    .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainNatureId()));

            if (CollectionUtils.isNotEmpty(basicEnums)) {
                businessGslrLossCollectVO.setGrainNatureName(basicEnums.get(0).getEnumname());
            }

            List<BasicEnum> basicEnums1 = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                    .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainVarietyId()));

            if (CollectionUtils.isNotEmpty(basicEnums1)) {
                businessGslrLossCollectVO.setGrainVarietyName(basicEnums1.get(0).getEnumname());
            }

            List<BasicEnum> basicEnums2 = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                    .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainGradeId()));

            if (CollectionUtils.isNotEmpty(basicEnums2)) {
                businessGslrLossCollectVO.setGrainGradeName(basicEnums2.get(0).getEnumname());
            }


            List<BasicWarehouse> basicWarehouses = basicWarehouseMapper.selectList(new LambdaQueryWrapper<BasicWarehouse>()
                    .eq(BasicWarehouse::getWarehouseId, businessGslrLossCollect.getWarehouseId()));

            if (CollectionUtils.isNotEmpty(basicWarehouses)) {
                businessGslrLossCollectVO.setWarehouseName(basicWarehouses.get(0).getWarehouseName());
            }

            List<BasicStorehouse> basicStorehouses = basicStorehouseMapper.selectList(new LambdaQueryWrapper<BasicStorehouse>()
                    .eq(BasicStorehouse::getStorehouseId, businessGslrLossCollect.getBasicStorehouseId()));

            if (CollectionUtils.isNotEmpty(basicStorehouses)) {
                businessGslrLossCollectVO.setStorehouseName(basicStorehouses.get(0).getStorehouseName());
            }

            businessGslrLossCollectVOS.add(businessGslrLossCollectVO);
        }
        return businessGslrLossCollectVOS;
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 损溢损失信息采集删除
     */
    @Override
    public int deleteData(String id) {
        BusinessGslrLossCollect lossCollect = businessGslrLossCollectMapper.selectById(id);
        if (lossCollect.getStatus() == 1) {
            throw new RuntimeException("损溢损失信息采集数据已上报，不能删除");
        }
        return businessGslrLossCollectMapper.deleteById(id);
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 损溢损失信息采集新增
     */
    @Override
    public int saveData(BusinessGslrLossCollectParam param) {

        BusinessGslrLossCollect businessGslrLossCollect = new BusinessGslrLossCollect();
        BeanUtils.copyProperties(param, businessGslrLossCollect);

        //损益单号
        BasicWarehouse basicWarehouse = basicWarehouseMapper.selectOne(Wrappers.lambdaQuery(BasicWarehouse.class)
                .eq(BasicWarehouse::getWarehouseId, param.getWarehouseId()));

        sequenceNumbers.putIfAbsent(basicWarehouse.getHwdm(), 0);
        // 增加当前货位的顺序号，并在达到最大值时重置为0
        int sequenceNumber = (sequenceNumbers.get(basicWarehouse.getHwdm()) + 1) % MAX_SEQUENCE_NUMBER;
        sequenceNumbers.put(basicWarehouse.getHwdm(), sequenceNumber);
        // 确保顺序号是3位数
        String formattedSequenceNumber = String.format("%03d", sequenceNumber);
        // 获取当前日期并格式化为yyyyMMdd
        String currentDate = LocalDate.now().format(DATE_FORMATTER);
        // 合并货位代码、日期和顺序号
        String sydh = basicWarehouse.getHwdm() + currentDate + formattedSequenceNumber;

        businessGslrLossCollect.setSydh(sydh);

        return businessGslrLossCollectMapper.insert(businessGslrLossCollect);


    }


    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 损溢损失信息采集修改
     */
    @Override
    public int updateData(BusinessGslrLossCollectParam param) {


        BusinessGslrLossCollect businessGslrLossCollect = new BusinessGslrLossCollect();
        BeanUtils.copyProperties(param, businessGslrLossCollect);

        return businessGslrLossCollectMapper.updateById(businessGslrLossCollect);
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 获取仓房名称信息
     */
    @Override
    public List<BasicStorehouse> getBasicStorehouse() {

        /*return basicStorehouseMapper.selectList(Wrappers.lambdaQuery(BasicStorehouse.class)
                .eq(BasicStorehouse::getOrgId,orgId));*/
        return basicStorehouseMapper.selectList(null);

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 获取货位名称信息
     */
    @Override
    public List<BasicWarehouse> getByWarehouse(String storehouseId) {

        return basicWarehouseMapper.selectList(Wrappers.lambdaQuery(BasicWarehouse.class)
                .eq(BasicWarehouse::getStorehouseId, storehouseId));

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 由货位id带出数据
     */
    @Override
    public WarehouseAndInfoVO getByWarehouseAndInfo(String warehouseId) {

        WarehouseAndInfoVO vo = new WarehouseAndInfoVO();

        if (StringUtils.isEmpty(warehouseId)) {
            throw new RuntimeException("请传入货位id");
        }
        //通过货位id查询粮情卡数据
        GrainSituationCard grainSituationCard = grainSituationCardMapper.selectOne(Wrappers.lambdaQuery(GrainSituationCard.class)
                .eq(GrainSituationCard::getWarehouseId, warehouseId));
        if (ObjectUtils.isEmpty(grainSituationCard)) {
            return null;
        }
        //获取入库净重
        vo.setNetWeightIn(grainSituationCard.getLssl().multiply(new BigDecimal("1000.0")));

        //获取出库净重(吨转千克)
        Date endDate = new Date();//现在时间
        Date startDate = grainSituationCard.getFormWarehouseTime();//封仓时间

        WarehouseAndInfoVO w = businessGslrLossCollectMapper.selectNetWeightOut(grainSituationCard.getGrainVarietyId(),
                grainSituationCard.getGrainNatureId(), grainSituationCard.getBasicStorehouseId(),
                grainSituationCard.getWarehouseId(), startDate, endDate);
        vo.setNetWeightOut(w.getNetWeightOut());


        //获取品种
        BasicEnum grainVariety = basicEnumMapper.selectOne(Wrappers.lambdaQuery(BasicEnum.class)
                .eq(BasicEnum::getEnumid, grainSituationCard.getGrainVarietyId()));
        vo.setGrainVarietyName(grainVariety.getEnumname());
        vo.setGrainVarietyId(grainSituationCard.getGrainVarietyId());
        //获取性质
        BasicEnum grainNature = basicEnumMapper.selectOne(Wrappers.lambdaQuery(BasicEnum.class)
                .eq(BasicEnum::getEnumid, grainSituationCard.getGrainNatureId()));
        vo.setGrainNatureName(grainNature.getEnumname());
        vo.setGrainNatureId(grainSituationCard.getGrainNatureId());
        //获取等级
        BasicEnum grainGrade = basicEnumMapper.selectOne(Wrappers.lambdaQuery(BasicEnum.class)
                .eq(BasicEnum::getEnumid, grainSituationCard.getGrainGradeId()));
        vo.setGrainGradeName(grainGrade.getEnumname());
        vo.setGrainGradeId(grainSituationCard.getGrainGradeId());

        return vo;
    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 分页展示上报
     */
    @Override
    public int report(String id) {

        return businessGslrLossCollectMapper.update(null, Wrappers.lambdaUpdate(BusinessGslrLossCollect.class)
                .eq(BusinessGslrLossCollect::getId, id)
                .set(BusinessGslrLossCollect::getStatus, 1));

    }

    /**
     * @author: hujianchun
     * @Description:
     * @param:
     * @return: 损溢损失信息采集详情
     */
    @Override
    public BusinessGslrLossCollectVO getDataById(String id) {

        BusinessGslrLossCollectVO businessGslrLossCollectVO = new BusinessGslrLossCollectVO();

        BusinessGslrLossCollect businessGslrLossCollect = businessGslrLossCollectMapper.selectById(id);
        if (ObjectUtils.isEmpty(businessGslrLossCollect)){
            throw new RuntimeException("暂无数据，id为"+id);
        }

       BeanUtils.copyProperties(businessGslrLossCollect,businessGslrLossCollectVO);


        //查询对应的库区名称
        OrgInfo orgInfo = orgInfoMapper.selectById(businessGslrLossCollect.getOrgId());
        businessGslrLossCollectVO.setOrgName(orgInfo.getOrgName());
        //查询对应的品种性质等级
        List<BasicEnum> basicEnums = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainNatureId()));

        if (CollectionUtils.isNotEmpty(basicEnums)) {
            businessGslrLossCollectVO.setGrainNatureName(basicEnums.get(0).getEnumname());
        }

        List<BasicEnum> basicEnums1 = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainVarietyId()));

        if (CollectionUtils.isNotEmpty(basicEnums1)) {
            businessGslrLossCollectVO.setGrainVarietyName(basicEnums1.get(0).getEnumname());
        }

        List<BasicEnum> basicEnums2 = basicEnumMapper.selectList(new LambdaQueryWrapper<BasicEnum>()
                .eq(BasicEnum::getEnumid, businessGslrLossCollect.getGrainGradeId()));

        if (CollectionUtils.isNotEmpty(basicEnums2)) {
            businessGslrLossCollectVO.setGrainGradeName(basicEnums2.get(0).getEnumname());
        }


        List<BasicWarehouse> basicWarehouses = basicWarehouseMapper.selectList(new LambdaQueryWrapper<BasicWarehouse>()
                .eq(BasicWarehouse::getWarehouseId, businessGslrLossCollect.getWarehouseId()));

        if (CollectionUtils.isNotEmpty(basicWarehouses)) {
            businessGslrLossCollectVO.setWarehouseName(basicWarehouses.get(0).getWarehouseName());
        }

        List<BasicStorehouse> basicStorehouses = basicStorehouseMapper.selectList(new LambdaQueryWrapper<BasicStorehouse>()
                .eq(BasicStorehouse::getStorehouseId, businessGslrLossCollect.getBasicStorehouseId()));

        if (CollectionUtils.isNotEmpty(basicStorehouses)) {
            businessGslrLossCollectVO.setStorehouseName(basicStorehouses.get(0).getStorehouseName());
        }

        return businessGslrLossCollectVO;

    }
}
