package com.chinaitop.depot.intelligent.priceApproval.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessFsAccount;
import com.chinaitop.depot.intelligent.priceApproval.model.*;
import com.chinaitop.depot.intelligent.priceApproval.service.BusinessFsStoragePriceApprovalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 财务监管-入库价格核定 前端控制器
 * </p>
 *
 * @author root
 * @since 2024-07-02
 */
@Api(tags = "财务监管-入库价格核定")
@RestController
@RequestMapping("/business-fs-storage-price-approval")
public class BusinessFsStoragePriceApprovalController {
    @Resource
    private BusinessFsStoragePriceApprovalService businessFsStoragePriceApprovalService;

    @GetMapping("/getContractInfo")
    @ApiOperation(value = "合同编号下拉", notes = "包含合同其它信息")
    public ResponseEntity<List<BusinessContract>> getContractInfo(Integer orgId) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getContractInfo(orgId));
    }

    @GetMapping("/getContractInfo/{id}")
    @ApiOperation(value = "合同信息", notes = "合同编号下拉id")
    public ResponseEntity<List<BusinessContractDetail>> getContractInfoList(@PathVariable("id") Integer id) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getContractInfoList(id));
    }

    @GetMapping("/getQualityInfo/{orgId}")
    @ApiOperation(value = "质检信息复选框", notes = "所有信息")
    public ResponseEntity<List<BusinessFsStoragePriceApprovalInspection>> getQualityInfo(@PathVariable("orgId") Integer orgId) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getQualityInfo(orgId));
    }

    @GetMapping("/getApprovalThree/{orgId}/{contractNumber}")
    @ApiOperation(value = "三账一致性", notes = "contractNumber合同单号")
    public ResponseEntity<List<BusinessFsStoragePriceApprovalThreeConsistence>> getApprovalThree(@PathVariable("orgId") Integer orgId, @PathVariable("contractNumber") String contractNumber) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getApprovalThree(orgId, contractNumber));
    }

    @GetMapping("/getApprovalThreeRKMX/{orgId}")
    @ApiOperation(value = "三账一致性-入库明细", notes = "入库明细")
    public ResponseEntity<List<DataCPZ>> getApprovalThreeRKMX(@PathVariable("orgId") Integer orgId) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getApprovalThreeRKMX(orgId));
    }

    @GetMapping("/getApprovalThreeKJPZ/{orgId}")
    @ApiOperation(value = "三账一致性-会计凭证号明细", notes = "会计凭证号明细")
    public ResponseEntity<List<BusinessFsAccount>> getApprovalThreeKJPZ(@PathVariable("orgId") Integer orgId) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getApprovalThreeKJPZ(orgId));
    }

    @GetMapping("/getApprovalSettle/{orgId}/{contractNumber}")
    @ApiOperation(value = "结算信息", notes = "列表")
    public ResponseEntity<List<BusinessFsStoragePriceApprovalSettle>> getApprovalSettle(@PathVariable("orgId") Integer orgId, @PathVariable("contractNumber") String contractNumber) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.getApprovalSettle(orgId, contractNumber));
    }

    @PostMapping("/save")
    @ApiOperation(value = "会计账新增", notes = "新增")
    public ResponseEntity save(@Validated @RequestBody BusinessFsStoragePriceApproval param) {
        businessFsStoragePriceApprovalService.saveData(param);
        return ResponseEntity.ok();
    }

    @GetMapping("/getById/{id}")
    @ApiOperation(value = "会计账详情", notes = "id获取详情")
    public ResponseEntity getListById(@PathVariable("id") Integer id) {
        BusinessFsStoragePriceApproval param = businessFsStoragePriceApprovalService.getListById(id);
        return ResponseEntity.ok(param);
    }

    @PostMapping("/update")
    @ApiOperation(value = "会计账修改", notes = "修改")
    public ResponseEntity update(@Validated @RequestBody BusinessFsStoragePriceApproval param) {
        businessFsStoragePriceApprovalService.updateData(param);
        return ResponseEntity.ok();
    }

    @GetMapping("/delete/{id}")
    @ApiOperation(value = "删除", notes = "id删除")
    public ResponseEntity delete(@PathVariable("id") Integer id) {
        businessFsStoragePriceApprovalService.deleteData(id);
        return ResponseEntity.ok();
    }

    @GetMapping("/getPageList")
    @ApiOperation(value = "分页列表", notes = "支持分页")
    public ResponseEntity<Page<BusinessFsStoragePriceApprovalPageResult>> getPageList(@Validated BusinessFsStoragePriceApprovalPage param) {
        return ResponseEntity.ok(businessFsStoragePriceApprovalService.selectPageList(param));
    }
}
