package com.chinaitop.depot.intelligent.pushs.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.chinaitop.depot.intelligent.common.ResponseEntity;
import com.chinaitop.depot.intelligent.devData.mapper.TLqDataLayerMapper;
import com.chinaitop.depot.intelligent.devData.mapper.TLqDataMapper;
import com.chinaitop.depot.intelligent.devData.model.TLqData;
import com.chinaitop.depot.intelligent.devData.model.TLqDataLayer;
import com.chinaitop.depot.intelligent.grainsituation.model.TPoints;
import com.chinaitop.depot.intelligent.pushs.service.GrainPushsService;
import com.chinaitop.depot.intelligent.utils.TempUtils;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import com.chinaitop.depot.intelligent.utils.VerificationSourceUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@SuppressWarnings("all")
public class GrainPushsServiceImpl implements GrainPushsService {
    final static Logger logger = LoggerFactory.getLogger(GrainPushsServiceImpl.class);

    @Resource
    private TLqDataMapper tLqDataMapper;
    @Resource
    private TLqDataLayerMapper lqDataLayerMapper;

    @Resource
    private RestTemplate restTemplate;

    @Value("${jsgjjsjUrl}")
    private String jsgjjsjUrl;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseEntity uploadGrainTemperatureData(List<TLqData> lqDataList) {
        if (ObjectUtils.isEmpty(lqDataList))
            return ResponseEntity.failed("空数据!");
        if (!VerificationSourceUtils.verificationSource(lqDataList.get(0).getDataSource())) {
            return ResponseEntity.failed("验证失败！");
        }
        //数据来源
        logger.info("lqData: {}", JSON.toJSONString(lqDataList));

        // 保存主表数据
        lqDataList.forEach(lqData -> {
            // 对象转字符串保存
            lqData.setTempPoints(JSON.toJSONString(lqData.getPoints()));
            // 计算最高,最低,平均粮温
            List<TPoints> points = lqData.getPoints();
            // 主键
            String lqDataId = UuidUtils.getCode();
            lqData.setId(lqDataId);
            // 最高
            BigDecimal max = points.stream().filter(TPoints -> !TempUtils.isBad(TPoints.getTemp())).map(TPoints::getTemp).reduce(BigDecimal.ZERO, BigDecimal::max);
            lqData.setTempMax(max);
            // 最低
            Optional<BigDecimal> minOptinal = points.stream().map(TPoints::getTemp).min(BigDecimal::compareTo);
            minOptinal.ifPresent(min -> lqData.setTempMin(min));
            // 平均
            BigDecimal wholeSum = points.stream().filter(TPoints -> !TempUtils.isBad(TPoints.getTemp())).map(TPoints::getTemp).reduce(BigDecimal.ZERO, BigDecimal::add);
            float doubleValue = wholeSum.floatValue();
            float wholeAvg = doubleValue / points.size();
            lqData.setTempAvg(new BigDecimal(wholeAvg == 0 ? 250 : wholeAvg));
            // 库存数量
            Map<String, Object> map = new HashMap<>();
            map.put("unitId", lqData.getOrgId());
            map.put("ch", lqData.getStorehouseCode());
            map.put("hwh", lqData.getHouse());
            Map<String, Object> mapR = tLqDataMapper.getKCSL(map);
            if (ObjectUtils.isNotEmpty(mapR)) {
                lqData.setKcsl(mapR.get("kcsl").toString());
                lqData.setFoodNature(mapR.get("foodNature").toString());
                lqData.setFoodVarietyName(mapR.get("foodVarietyName").toString());
            }
            // 计算->同上次相比平均粮温 1：相同；2：上升；3：下降
            Map<String, Object> tempSelect = new HashMap<>();
            if (ObjectUtils.isNotEmpty(lqData.getOrgId()))
                map.put("orgId", lqData.getOrgId());
            if (StringUtils.isNotEmpty(lqData.getOrgCode()))
                map.put("orgCode", lqData.getOrgCode());
            if (StringUtils.isNotEmpty(lqData.getStorehouseCode()))
                map.put("storehouseCode", lqData.getStorehouseCode());
            Float bigDecimal = tLqDataMapper.selectUpTemp(tempSelect);
            if (ObjectUtils.isNotEmpty(bigDecimal)) {
                float oldTemp = bigDecimal.floatValue();
                float newTemp = lqData.getInTemp().floatValue();
                if (newTemp > oldTemp) // 新数据大于老数据
                    lqData.setCompareLastTempAvg(2);
                else if (newTemp < oldTemp) // 新数据小于老数据
                    lqData.setCompareLastTempAvg(3);
                else
                    lqData.setCompareLastTempAvg(1);
            } else {
                lqData.setCompareLastTempAvg(1);
            }
            // 保存
            tLqDataMapper.insert(lqData);

            this.gjjjk(lqDataId, Integer.valueOf(lqData.getOrgId()), "i");

            // 保存子表数据
            Map<String, List<TPoints>> layerGroupV = points.stream().collect(Collectors.groupingBy(t -> t.getZ()));
            layerGroupV.forEach((k, v) -> {
                TLqDataLayer layer = new TLqDataLayer();
                layer.setId(UuidUtils.getCode());
                layer.setOrgId(lqData.getOrgId());
                layer.setLqId(lqDataId);
                layer.setLhouse(lqData.getStorehouseCode());
                layer.setLtime(lqData.getGatherTime());
                layer.setLayernumber(Integer.parseInt(k));
                layer.setLz(k);
                // 每层最高
                BigDecimal lsyerMax = v.stream().filter(TPoints -> !TPoints.getTemp().equals(new BigDecimal("250.0"))).map(TPoints::getTemp).reduce(BigDecimal.ZERO, BigDecimal::max);
                layer.setLmax(lsyerMax);
                // 每层最低
                Optional<BigDecimal> vMin = v.stream().map(TPoints::getTemp).min(BigDecimal::compareTo);
                vMin.ifPresent(lsyerMin -> layer.setLmin(lsyerMin));
                // 每层平均温
                BigDecimal lsyerSums = v.stream().filter(TPoints -> !TPoints.getTemp().equals(new BigDecimal("250.0"))).map(TPoints::getTemp).reduce(BigDecimal.ZERO, BigDecimal::add);
                float lsyerSum = lsyerSums.floatValue();
                float lsyerAvg = 0;
                if (v.size() > 0) {
                    lsyerAvg = lsyerSum / v.size();
                }
                layer.setLavg(new BigDecimal(lsyerAvg == 0 ? 250 : lsyerAvg));
                layer.setDataSource(lqData.getDataSource());
                lqDataLayerMapper.insert(layer);
            });
        });
        return ResponseEntity.ok();
    }

    /**
     * 国家局数据
     */
    public void gjjjk(String id, Integer orgId, String czbz) {

        String tableName = "ODS_UP_WSDJCXXJK";
        /**
         * restTemplate调用
         */
        //String url = "http://localhost:9022/recPublish/sendMessage";//
        //发送post请求，并打印结果，以String类型接收响应结果JSON字符串

        MultiValueMap<String, Object> sendMap = new LinkedMultiValueMap<>();
        sendMap.add("tableName", tableName);
        sendMap.add("czbz", czbz);
        sendMap.add("ywId", id);
        sendMap.add("orgId", orgId);

//        restTemplate.postForObject(jsgjjsjUrl, sendMap, String.class);


    }

}
